//+------------------------------------------------------------------+
//|                                    Macd_with_EMA_BDv09_12_22.mq4 |
//|                                                             Jeff |
//|                                                  wolfboy@att.net |
//+------------------------------------------------------------------+
#property copyright "Jeff"
#property link      "wolfboy@att.net"

//Macd_with_EMA_BDv9_07_19:   Modified the Macd_with_EMA_BDv8_12_31 indicator to show the 
//                            macd histogram in four colors.
//Macd_with_EMA-BDv09_12_22:  Fixed problem with very first green bar being dark green (now is
//                            light green).

#property indicator_separate_window
#property indicator_buffers 7
#property indicator_color1 LimeGreen
#property indicator_color2 DarkGreen
#property indicator_color3 Red
#property indicator_color4 Maroon
#property indicator_color5 CLR_NONE
#property indicator_color6 Black
#property indicator_color7 Gold
#property indicator_width1 4
#property indicator_width2 4
#property indicator_width3 4
#property indicator_width4 4
#property indicator_width5 1
#property indicator_width6 2
#property indicator_width7 2
#property indicator_level1 0.0
#property indicator_levelcolor DimGray
#property indicator_levelstyle 0

extern string  TimeFrame     = "Current time frame";
extern int     FastEMA       = 10;
extern int     SlowEMA       = 20;
extern int     SignalEMA     = 1;
extern int     MAofSignalPer = 7;

//---- indicator buffers
double MacdBuffer[];
double SignalBuffer[];
double MAofSignalBuffer[];
double green_buffer[];
double DarkGreen_buffer[];
double red_buffer[];
double Maroon_buffer[];

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
   {
   IndicatorBuffers(7);
   
//---- drawing settings
   SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID);
   SetIndexStyle(1,DRAW_HISTOGRAM,STYLE_SOLID);
   SetIndexStyle(2,DRAW_HISTOGRAM,STYLE_SOLID);
   SetIndexStyle(3,DRAW_HISTOGRAM,STYLE_SOLID);         
   SetIndexStyle(4,DRAW_NONE);
   SetIndexStyle(5,DRAW_LINE,STYLE_SOLID);
   SetIndexStyle(6,DRAW_LINE,STYLE_DOT);   
   SetIndexDrawBegin(1,SignalEMA);
   IndicatorDigits(Digits+1);
   
//---- indicator buffers mapping
   SetIndexBuffer(0,green_buffer);
   SetIndexBuffer(1,DarkGreen_buffer);
   SetIndexBuffer(2,red_buffer);
   SetIndexBuffer(3,Maroon_buffer);
   SetIndexBuffer(4,MacdBuffer);      
   SetIndexBuffer(5,SignalBuffer);
   SetIndexBuffer(6,MAofSignalBuffer);
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);

   
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName(timeFrameToString(timeFrame)+" MACD in color ("+FastEMA+", "+SlowEMA+", "+SignalEMA+", "+MAofSignalPer+")");
   SetIndexLabel(4,"MACD in color");
   SetIndexLabel(5,"Signal");
   SetIndexLabel(6,"MAofSignal");
   
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { green_buffer[0] = limit+1; return(0); }
   
            if (timeFrame!=Period())
            {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for (int i=limit; i>=0; i--)
               {
                   int y = iBarShift(NULL,timeFrame,Time[i]);   
                      green_buffer[i]     = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,0,y);
                      DarkGreen_buffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,1,y);
                      red_buffer[i]       = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,2,y);
                      Maroon_buffer[i]    = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,3,y);
                      MacdBuffer[i]       = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,4,y);
                      SignalBuffer[i]     = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,5,y);
                      MAofSignalBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,6,y);
               }
               return(0);
            }
            
   double MACD;
   counted_bars=IndicatorCounted();
   
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
   
//---- macd
   for(i=0; i<limit; i++)
      MacdBuffer[i]=iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)-
                    iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
      
//---- signal line
   for(i=0; i<limit; i++)
      SignalBuffer[i]=iMAOnArray(MacdBuffer,Bars,SignalEMA,0,MODE_EMA,i);
      
//---- MA of signal
   for(i=0; i<limit; i++)
      MAofSignalBuffer[i]=iMAOnArray(SignalBuffer,Bars,MAofSignalPer,0,MODE_EMA,i);
      
   for (i = Bars - Max (counted_bars-1, 1); i>=0; i--) 
      {
      MACD=MacdBuffer[i];
           
      if(MACD>0) 
         {
         if (MACD>MacdBuffer[i+1])
            {
            green_buffer[i] = MACD;
            DarkGreen_buffer[i] = 0;
            }
         else 
            {
            green_buffer[i] = 0;
            DarkGreen_buffer[i] = MACD;
            }
         }
      else 
         {
         if (MACD<MacdBuffer[i+1])
            {
            red_buffer[i] = 0;
            Maroon_buffer[i] = MACD;
            }
         else 
            {
            red_buffer[i] = MACD;
            Maroon_buffer[i] = 0;
            } 
         }
      }
      
//---- done
   return(0);
  }

//+------------------------------------------------------------------+
int Max (int val1, int val2) {
  if (val1 > val2)  return(val1);
  return(val2);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(MathMax(Period(),StrToDouble(tfs)));
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}






